# Copyright 2005-2012 Canonical Limited.  All rights reserved.

import json
from datetime import datetime
from base64 import b64encode

from landscape_api.base import API, UsageError
from mocker import MockerTestCase


class APITest(MockerTestCase):

    def setUp(self):
        super(APITest, self).setUp()
        self.api = API("http://localhost", "access", "secret")
        self.api._run_query = self.run_query
        self.calls = []

    def run_query(self, *args):
        self.calls.append(args)
        return json.dumps("data")

    def assertCallEqual(self, action, params):
        self.assertEqual(
            [("access", "secret", action, params, "http://localhost", None)],
            self.calls)

    def test_get_computers(self):
        """
        L{API.get_computers} makes a C{GetComputers} call with the C{run_query}
        function.
        """
        self.api.get_computers("id:1", 10, 5)
        self.assertCallEqual(
            "GetComputers",
            {"query": "id:1", "limit": "10", "offset": "5"})

    def test_get_pending_computers(self):
        """
        L{API.get_pending_computers} makes a C{GetPendingComputers} call with
        the C{run_query} function.
        """
        self.api.get_pending_computers()
        self.assertCallEqual(
            "GetPendingComputers", {})

    def test_get_activities(self):
        """
        L{API.get_activities} makes a C{GetActivities} call with the
        C{run_query} function.
        """
        self.api.get_activities("id:1", 10, 5)
        self.assertCallEqual(
            "GetActivities",
            {"query": "id:1", "limit": "10", "offset": "5"})

    def test_approve_activities(self):
        """
        L{API.approve_activities} makes a C{ApproveActivities} call with the
        C{run_query} function.
        """
        self.api.approve_activities("status:unapproved")
        self.assertCallEqual(
            "ApproveActivities", {"query": "status:unapproved"})

    def test_cancel_activities(self):
        """
        L{API.cancel_activities} makes a C{CancelActivities} call with the
        C{run_query} function.
        """
        self.api.cancel_activities("status:unapproved")
        self.assertCallEqual(
            "CancelActivities", {"query": "status:unapproved"})

    def test_add_tags_to_computers(self):
        """
        L{API.add_tags_to_computers} makes a C{AddTagsToComputers} call with
        the C{run_query} function, turning the list of tags to proper C{tags.#}
        parameters.
        """
        self.api.add_tags_to_computers("tag:server", ["web"])
        self.assertCallEqual(
            "AddTagsToComputers", {"query": "tag:server", "tags.1": "web"})

    def test_remove_tags_from_computers(self):
        """
        L{API.remove_tags_from_computers} makes a C{RemoveTagsFromComputers}
        call with the C{run_query} function, turning the list of tags to proper
        C{tags.#} parameters.
        """
        self.api.remove_tags_from_computers("tag:server", ["web"])
        self.assertCallEqual(
            "RemoveTagsFromComputers",
            {"query": "tag:server", "tags.1": "web"})

    def test_get_scripts(self):
        """
        L{API.get_scripts} makes a C{GetScripts} call with the C{run_query}
        function.
        """
        self.api.get_scripts(10, 5)
        self.assertCallEqual("GetScripts", {"limit": "10", "offset": "5"})

    def test_execute_script(self):
        """
        L{API.execute_script} makes a C{ExecuteScript} call with the
        C{run_query} function.
        """
        scheduled = datetime(2021, 10, 30)
        self.api.execute_script("tag:server", 3, deliver_after=scheduled)
        self.assertCallEqual(
            "ExecuteScript",
            {"query": "tag:server", "script_id": "3",
             "deliver_after": "2021-10-30T00:00:00Z"})

    def test_execute_script_deliver_after_as_string(self):
        """
        If the C{deliver_after} parameter of L{API.execute_script} is a string,
        it's passed as-is to the query.
        """
        self.api.execute_script("tag:server", 3,
                                deliver_after="2021-10-30T00:00:00Z")
        self.assertCallEqual(
            "ExecuteScript",
            {"query": "tag:server", "script_id": "3",
             "deliver_after": "2021-10-30T00:00:00Z"})

    def test_get_packages(self):
        """
        L{API.get_packages} makes a C{GetPackages} call with the C{run_query}
        function.
        """
        self.api.get_packages("id:1")
        self.assertCallEqual("GetPackages", {"query": "id:1"})

    def test_get_packages_with_search(self):
        """
        L{API.get_packages} takes an optional search argument to restrict the
        packages returned.
        """
        self.api.get_packages("id:1", "python")
        self.assertCallEqual(
            "GetPackages",
            {"query": "id:1", "search": "python"})

    def test_get_packages_with_limit(self):
        """
        L{API.get_packages} takes an optional limit argument to control the
        number of packages returned.
        """
        self.api.get_packages("id:1", limit=10)
        self.assertCallEqual(
            "GetPackages",
            {"query": "id:1",
             "limit": "10"})

    def test_get_packages_with_offset(self):
        """
        L{API.get_packages} takes an optional offset argument to navigate in
        the list of results.
        """
        self.api.get_packages("id:1", offset=10)
        self.assertCallEqual(
            "GetPackages",
            {"query": "id:1",
             "offset": "10"})

    def test_get_packages_with_limit_and_offset(self):
        """
        L{API.get_packages} can use both limit and offset to navigate in the
        list of results.
        """
        self.api.get_packages("id:1", "python", limit=10, offset=5)
        self.assertCallEqual(
            "GetPackages",
            {"query": "id:1", "search": "python",
             "limit": "10", "offset": "5"})

    def test_get_packages_without_available(self):
        """
        L{API.get_packages} takes an optional available to restrict the
        type of packages returned.
        """
        self.api.get_packages("id:1", available=False)
        self.assertCallEqual("GetPackages", {"query": "id:1",
                                             "available": "false"})

    def test_get_packages_without_installed(self):
        """
        L{API.get_packages} takes an optional installed to restrict the
        type of packages returned.
        """
        self.api.get_packages("id:1", installed=False)
        self.assertCallEqual("GetPackages", {"query": "id:1",
                                             "installed": "false"})

    def test_get_packages_without_upgrades(self):
        """
        L{API.get_packages} takes an optional upgrades to
        restrict the type of packages returned.
        """
        self.api.get_packages("id:1", upgrade=False)
        self.assertCallEqual(
            "GetPackages", {"query": "id:1", "upgrade": "false"})

    def test_install_packages(self):
        """
        L{API.install_packages} makes a C{InstallPackages} call with the
        C{run_query} function.
        """
        self.api.install_packages("id:1", ["python", "postgresql-8.4"])
        self.assertCallEqual(
            "InstallPackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4"})

    def test_install_packages_with_deliver_after(self):
        """
        L{API.install_packages} makes a C{InstallPackages} call with the
        C{run_query} function.
        """
        scheduled = datetime(2021, 10, 30, 4, 15, 15)
        self.api.install_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "InstallPackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_install_packages_with_deliver_after_as_string(self):
        """
        L{API.install_packages} makes a C{InstallPackages} call with the
        C{run_query} function.
        """
        scheduled = "2021-10-30T04:15:15Z"
        self.api.install_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "InstallPackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_remove_packages(self):
        """
        L{API.remove_packages} makes a C{RemovePackages} call with the
        C{run_query} function.
        """
        self.api.remove_packages("id:1", ["python", "postgresql-8.4"])
        self.assertCallEqual(
            "RemovePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4"})

    def test_remove_packages_with_deliver_after(self):
        """
        L{API.remove_packages} makes a C{RemovePackages} call with the
        C{run_query} function.
        """
        scheduled = datetime(2021, 10, 30, 4, 15, 15)
        self.api.remove_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "RemovePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_remove_packages_with_deliver_after_as_string(self):
        """
        L{API.remove_packages} makes a C{RemovePackages} call with the
        C{run_query} function.
        """
        scheduled = "2021-10-30T04:15:15Z"
        self.api.remove_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "RemovePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_upgrade_packages(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        self.api.upgrade_packages("id:1")
        self.assertCallEqual("UpgradePackages", {"query": "id:1"})

    def test_upgrade_packages_as_list(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        self.api.upgrade_packages("id:1", ["python", "postgresql-8.4"])
        self.assertCallEqual(
            "UpgradePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4"})

    def test_upgrade_packages_with_deliver_after(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        scheduled = datetime(2021, 10, 30, 4, 15, 15)
        self.api.upgrade_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "UpgradePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_upgrade_packages_with_deliver_after_as_string(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        scheduled = "2021-10-30T04:15:15Z"
        self.api.upgrade_packages("id:1", ["python", "postgresql-8.4"],
            deliver_after=scheduled)
        self.assertCallEqual(
            "UpgradePackages",
            {"query": "id:1",
             "packages.1": "python",
             "packages.2": "postgresql-8.4",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_upgrade_packages_security_only(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        self.api.upgrade_packages("id:1", security_only=True)
        self.assertCallEqual(
            "UpgradePackages",
            {"query": "id:1",
             "security_only": "true"})

    def test_get_package_profiles(self):
        """
        L{API.get_package_profiles} makes a C{GetPackageProfiles} call
        with the C{run_query} function.
        """
        self.api.get_package_profiles()
        self.assertCallEqual(
            "GetPackageProfiles", {})

    def test_get_package_profiles_with_name(self):
        """
        L{API.get_package_profiles} makes a C{GetPackageProfiles} call
        with the C{run_query} function adding optional C{name}
        parameter.
        """
        self.api.get_package_profiles(names=["MyAwesomeProfile"])
        self.assertCallEqual(
            "GetPackageProfiles", {"names.1": u"MyAwesomeProfile"})

    def test_copy_package_profile(self):
        """
        L{API.copy_package_profile} makes a C{CopyPackageProfile} call
        with the C{run_query} function passing the C{name} argument.
        """
        self.api.copy_package_profile("MyAwesomeProfile")
        self.assertCallEqual(
            "CopyPackageProfile", {"name": u"MyAwesomeProfile"})

    def test_copy_package_profile_with_destinationname(self):
        """
        L{API.copy_package_profile} makes a C{CopyPackageProfile} call
        with the C{run_query} function adding optional C{destination_name}
        parameter.
        """
        self.api.copy_package_profile("MyAwesomeProfile", "NewProfileName")
        self.assertCallEqual(
            "CopyPackageProfile", {"name": u"MyAwesomeProfile",
                                   "destination_name": u"NewProfileName"})

    def test_create_package_profile(self):
        """
        L{API.create_package_profile} makes a C{CreatePackageProfile} call.
        """
        self.api.create_package_profile("foobar", "description")
        self.assertCallEqual(
            "CreatePackageProfile",
            {"name": u"foobar",
             "description": "description"})

    def test_create_package_profile_with_computer(self):
        """
        L{API.create_package_profile} specifies the C{source_computer_id}
        parameter when it is passed in.
        """
        self.api.create_package_profile("foobar", "description",
                                        source_computer_id=599)
        self.assertCallEqual(
            "CreatePackageProfile",
            {"name": u"foobar",
             "description": "description",
             "source_computer_id": "599"})

    def test_create_package_profile_with_material(self):
        """
        L{API.create_package_profile} specifies the C{material} parameter when
        it is passed in.
        """
        self.api.create_package_profile("foobar", "description",
                                        material="heya")
        self.assertCallEqual(
            "CreatePackageProfile",
            {"name": u"foobar",
             "description": "description",
             "material": "heya"})

    def test_associate_package_profile(self):
        """
        L{API.associate_package_profile} invokes the AssociatePackageProfile
        API.
        """
        self.api.associate_package_profile("foobar",
                                           tags=["laptop", "desktop"])
        self.assertCallEqual(
            "AssociatePackageProfile",
            {"name": u"foobar",
             "tags.1": "laptop",
             "tags.2": "desktop"})

    def test_remove_package_profile(self):
        """
        L{API.remove_package_profile} invokes the RemovePackageProfile API.
        """
        self.api.remove_package_profile("foobar")
        self.assertCallEqual(
            "RemovePackageProfile",
            {"name": u"foobar"})

    def test_modify_package_profile(self):
        """
        L{API.modify_package_profile} invokes the ModifyPackageProfile
        API.
        """
        self.api.modify_package_profile(
            "foobar", ["depends python-twisted", "conflicts python < 2.5"],
            ["conflicts wget", "depends extremetuxracer"])
        self.assertCallEqual(
            "ModifyPackageProfile",
            {"name": u"foobar",
             "add_constraints.1": "depends python-twisted",
             "add_constraints.2": "conflicts python < 2.5",
             "remove_constraints.1": "conflicts wget",
             "remove_constraints.2": "depends extremetuxracer",
            })

    def test_register_provisioning_server(self):
        """
        L{API.register_provisioning_servers} makes a
        C{RegisterProvisioningServer} call with the C{run_query} function.
        """
        self.api.register_provisioning_server(u"servername", u"servertitle",
            "http://endpoint:1234", "credentials:credentials")
        self.assertCallEqual(
            "RegisterProvisioningServer",
            {"name": u"servername",
             "title": u"servertitle",
             "endpoint": "http://endpoint:1234",
             "credentials": "credentials:credentials"})

    def test_get_provisioning_servers(self):
        """
        L{API.get_provisioning_servers} makes a C{GetProvisioningServers} call
        with the C{run_query} function.
        """
        self.api.get_provisioning_servers()
        self.assertCallEqual(
            "GetProvisioningServers", {})

    def test_provision_system(self):
        """
        L{API.provision_system} makes a C{ProvisionSystem} call with the
        C{run_query} function.
        """
        self.api.provision_system(u"systemtitle", "hostname",
                                  "11:22:33:44:55:66", "precise", "i386",
                                  u"servername")
        self.assertCallEqual(
            "ProvisionSystem",
            {"server_name": "servername",
             "title": "systemtitle",
             "hostname": "hostname",
             "mac_address": "11:22:33:44:55:66",
             "release": "precise",
             "arch": "i386"})

    def test_provision_system_with_tags(self):
        """
        L{API.provision_system} makes a C{ProvisionSystem} call providing
        a list of tags.
        """
        self.api.provision_system(u"systemtitle", "hostname",
                                  "11:22:33:44:55:66", "precise", "i386",
                                  u"servername", ["tag1", "tag2"])
        self.assertCallEqual(
            "ProvisionSystem",
            {"server_name": "servername",
             "title": "systemtitle",
             "hostname": "hostname",
             "mac_address": "11:22:33:44:55:66",
             "release": "precise",
             "arch": "i386",
             "tags.1": "tag1",
             "tags.2": "tag2"})

    def test_provision_system_with_cloud_config_file(self):
        """
        L{API.provision_system} makes a C{ProvisionSystem} call with the
        C{run_query} function adding optional C{cloud_config} parameter.
        """
        content = "#cloud-config\n"
        content += "# Upgrade the instance on first boot\n"
        content += "(ie run apt-get upgrade)\n#\n"
        content += "# Default: false\n#\n"
        content += "apt_upgrade: true\n"
        cloud_config_file = self.makeFile(content)

        with open(cloud_config_file, "rb") as cloud_script:
            script_content = cloud_script.read()
        cloud_config_b64 = b64encode(script_content)

        self.api.provision_system_from_file(
            u"systemtitle", "hostname", "11:22:33:44:55:66", "precise", "i386",
            server_name=u"servername", cloud_config_file=cloud_config_file)
        self.assertCallEqual(
            "ProvisionSystem",
            {"server_name": "servername",
             "title": "systemtitle",
             "hostname": "hostname",
             "mac_address": "11:22:33:44:55:66",
             "release": "precise",
             "arch": "i386",
             "cloud_config": cloud_config_b64})

    def test_provision_system_with_bad_cloud_config_file(self):
        e = self.assertRaises(
            UsageError,
            self.api.provision_system_from_file,
            u"servername", u"systemtitle", "hostname",
            "11:22:33:44:55:66", "precise", "i386",
            cloud_config_file="nonexistent_cloud_config_file")
        self.assertIn("Unable to find cloud-config-file at "
                      "nonexistent_cloud_config_file",
                      e.stderr)

    def test_terminate_system(self):
        """
        L{API.terminate_system} makes a C{TerminateSystem} call with the
        C{run_query} function.
        """
        self.api.terminate_system("hostname")
        self.assertCallEqual(
            "TerminateSystem",
            {"hostname": u"hostname"})

    def test_upgrade_packages_security_only_deliver_after(self):
        """
        L{API.upgrade_packages} makes a C{UpgradePackages} call with the
        C{run_query} function.
        """
        scheduled = datetime(2021, 10, 30, 4, 15, 15)
        self.api.upgrade_packages("id:1", security_only=True,
            deliver_after=scheduled)
        self.assertCallEqual(
            "UpgradePackages",
            {"query": "id:1",
             "security_only": "true",
             "deliver_after": "2021-10-30T04:15:15Z"})

    def test_import_gpg_key(self):
        """
        L{API.import_gpg_key} makes a C{ImportGPGKey} call with the
        C{run_query} function.
        """
        keyfile = self.makeFile("ABBADDEF")
        self.api.import_gpg_key_from_file("key", keyfile)
        self.assertCallEqual(
            "ImportGPGKey",
            {"name": "key",
             "material": "ABBADDEF"})

    def test_remove_gpg_key(self):
        """
        L{API.remove_gpg_key} makes a C{RemoveGPGKey} call with the
        C{run_query} function.
        """
        self.api.remove_gpg_key("key")
        self.assertCallEqual(
            "RemoveGPGKey",
            {"name": "key"})

    def test_get_gpg_keys(self):
        """
        L{API.get_gpg_keys} makes a C{GetGPGKeys} call with the C{run_query}
        function.
        """
        self.api.get_gpg_keys()
        self.assertCallEqual("GetGPGKeys", {})

    def test_get_gpg_keys_with_names_as_list(self):
        """
        L{API.get_gpg_keys} makes a C{GetGPGKeys} call with the C{run_query}
        function.
        """
        self.api.get_gpg_keys(names=["key1", "key2"])
        self.assertCallEqual("GetGPGKeys",
                             {"names.1": "key1", "names.2": "key2"})

    def test_create_distribution(self):
        """
        L{API.create_distribution} makes a C{CreateDistribution} call with the
        C{run_query} function.
        """
        self.api.create_distribution("dog")
        self.assertCallEqual(
            "CreateDistribution",
            {"name": "dog"})

    def test_remove_distribution(self):
        """
        L{API.remove_distribution} makes a C{RemoveDistribution} call with the
        C{run_query} function.
        """
        self.api.remove_distribution("dog")
        self.assertCallEqual(
            "RemoveDistribution",
            {"name": "dog"})

    def test_get_distributions(self):
        """
        L{API.get_distributions} makes a C{GetDistributions} call with the
        C{run_query} function.
        """
        self.api.get_distributions()
        self.assertCallEqual(
            "GetDistributions",
            {})

    def test_get_distributions_names_as_list(self):
        """
        L{API.get_distributions} makes a C{GetDistributions} call with the
        C{run_query} function.
        """
        self.api.get_distributions(names=["dog", "cat"])
        self.assertCallEqual(
            "GetDistributions",
            {"names.1": "dog",
             "names.2": "cat"})

    def test_create_series(self):
        """
        L{API.create_series} makes a C{CreateSeries} call with the C{run_query}
        function.
        """
        self.api.create_series("dog", "cat-distribution")
        self.assertCallEqual(
            "CreateSeries",
            {"name": "dog",
             "distribution": "cat-distribution"})

    def test_create_series_with_pocket(self):
        """
        L{API.create_series} makes a C{CreateSeries} call with the C{run_query}
        function, with optional pocket parameters.
        """
        self.api.create_series("natty", "ubuntu",
                               pockets=["release", "dev"],
                               components=["main", "universe"],
                               architectures=["amd64"],
                               gpg_key="my-key",
                               mirror_uri="http://example.com",
                               mirror_gpg_key="mirror-key")

        self.assertCallEqual(
            "CreateSeries",
            {"name": "natty",
             "distribution": "ubuntu",
             "pockets.1": "release",
             "pockets.2": "dev",
             "components.1": "main",
             "components.2": "universe",
             "architectures.1": "amd64",
             "gpg_key": "my-key",
             "mirror_uri": "http://example.com",
             "mirror_gpg_key": "mirror-key"})

    def test_create_series_with_pocket_and_series(self):
        """
        L{API.create_series} makes a C{CreateSeries} call with the C{run_query}
        function, with optional pocket parameters and C{MirrorSeries}.
        """
        self.api.create_series("natty", "ubuntu",
                               pockets=["release", "dev"],
                               components=["main", "universe"],
                               architectures=["amd64"],
                               gpg_key="my-key",
                               mirror_uri="http://example.com",
                               mirror_series="natty",
                               mirror_gpg_key="mirror-key")

        self.assertCallEqual(
            "CreateSeries",
            {"name": "natty",
             "distribution": "ubuntu",
             "pockets.1": "release",
             "pockets.2": "dev",
             "components.1": "main",
             "components.2": "universe",
             "architectures.1": "amd64",
             "gpg_key": "my-key",
             "mirror_uri": "http://example.com",
             "mirror_series": "natty",
             "mirror_gpg_key": "mirror-key"})

    def test_remove_series(self):
        """
        L{API.remove_series} makes a C{RemoveSeries} call with the C{run_query}
        function.
        """
        self.api.remove_series("dog", "cat-distribution")
        self.assertCallEqual(
            "RemoveSeries",
            {"name": "dog",
             "distribution": "cat-distribution"})

    def test_derive_series(self):
        """
        L{API.derive_series} makes a C{DeriveSeries} call with the C{run_query}
        function.
        """
        self.api.derive_series("dog", "woof", "cat-distribution")
        self.assertCallEqual(
            "DeriveSeries",
            {"name": "dog",
             "origin": "woof",
             "distribution": "cat-distribution"})

    def test_create_pocket(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket("name", "series", "distribution",
                                          ["component1", "component2"],
                                          ["arch1", "arch2"],
                                          "mirror", "sign-key")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key"})

    def test_create_pocket_with_mirror_uri(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket("name", "series", "distribution",
                                          ["component1", "component2"],
                                          ["arch1", "arch2"],
                                          "mirror", "sign-key",
                                          mirror_uri="mirror_uri")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "mirror_uri": "mirror_uri"})

    def test_create_pocket_with_mirror_suite(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function with specified C{mirror_suite} parameter.
        """
        self.api.create_pocket(
            "name", "series", "distribution",
            ["component1", "component2"],
            ["arch1", "arch2"],
            "mirror", "sign-key",
            mirror_uri="mirror_uri",
            mirror_suite="mirror_suite")

        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "mirror_uri": "mirror_uri",
             "mirror_suite": "mirror_suite"})

    def test_create_pocket_with_mirror_gpg_key(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket(
            "name", "series", "distribution",
            ["component1", "component2"],
            ["arch1", "arch2"],
            "mirror", "sign-key",
            mirror_gpg_key="mirror_key")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "mirror_gpg_key": "mirror_key"})

    def test_create_pocket_with_pull_pocket(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket("name", "series", "distribution",
                               ["component1", "component2"],
                               ["arch1", "arch2"],
                               "mirror", "sign-key",
                               pull_pocket="pull_pocket")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "pull_pocket": "pull_pocket"})

    def test_create_pocket_with_pull_series(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket("name", "series", "distribution",
                               ["component1", "component2"],
                               ["arch1", "arch2"],
                               "mirror", "sign-key",
                               pull_series="pull_series")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "pull_series": "pull_series"})

    def test_create_pocket_with_filter_type(self):
        """
        L{API.create_pocket} makes a C{CreatePocket} call with the C{run_query}
        function.
        """
        self.api.create_pocket("name", "series", "distribution",
                               ["component1", "component2"],
                               ["arch1", "arch2"],
                               "mirror", "sign-key",
                               pull_pocket="pull_pocket",
                               filter_type="whitelist")
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "components.2": "component2",
             "architectures.1": "arch1", "architectures.2": "arch2",
             "mode": "mirror", "gpg_key": "sign-key",
             "pull_pocket": "pull_pocket", "filter_type": "whitelist"})

    def test_create_pocket_with_upload_allow_unsigned(self):
        """
        L{API.create_pocket} accepts an optional C{upload_allow_unsigned}
        parameter.
        """
        self.api.create_pocket("name", "series", "distribution",
                               ["component1"], ["arch1"], "upload", "sign-key",
                               upload_allow_unsigned=True)
        self.assertCallEqual(
            "CreatePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "components.1": "component1", "architectures.1": "arch1",
             "mode": "upload", "gpg_key": "sign-key",
             "upload_allow_unsigned": "true"})

    def test_sync_mirror_pocket(self):
        """
        L{API.sync_mirror_pocket} makes a C{SyncMirrorPocket} call with the
        C{run_query} function.
        """
        self.api.sync_mirror_pocket("name", "series", "distribution")
        self.assertCallEqual(
            "SyncMirrorPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution"})

    def test_pull_packages_to_pocket(self):
        """
        L{API.pull_packages_to_pocket} makes a C{SyncPullPocket} call with the
        C{run_query} function.
        """
        self.api.pull_packages_to_pocket("name", "series", "distribution")
        self.assertCallEqual(
            "PullPackagesToPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution"})

    def test_remove_packages_from_pocket(self):
        """
        L{API.remove_packages_from_pocket} makes a
        C{RemovePackagesFromPocket} call with the C{run_query} function.
        """
        self.api.remove_packages_from_pocket(
            "name", "series", "distribution", ["foo", "bar"])
        self.assertCallEqual(
            "RemovePackagesFromPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "packages.1": "foo", "packages.2": "bar"})

    def test_diff_pull_pocket(self):
        """
        L{API.diff_pull_pocket} makes a C{DiffPullPocket} call with the
        C{run_query} function.
        """
        self.api.diff_pull_pocket("name", "series", "distribution")
        self.assertCallEqual(
            "DiffPullPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution"})

    def test_list_pocket(self):
        """
        L{API.list_pocket} makes a C{ListPocket} call with the C{run_query}
        function.
        """
        self.api.list_pocket("name", "series", "distribution")
        self.assertCallEqual(
            "ListPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution"})

    def test_remove_pocket(self):
        """
        L{API.remove_pocket} makes a C{RemovePocket} call with the C{run_query}
        function.
        """
        self.api.remove_pocket("name", "series", "distribution")
        self.assertCallEqual(
            "RemovePocket",
            {"name": "name", "series": "series",
             "distribution": "distribution"})

    def test_add_uploader_gpg_keys_to_pocket(self):
        """
        L{API.add_uploader_gpg_keys_to_pocket} makes a
        C{AddUploaderGPGKeysToPocket} call with the C{run_query} function.
        """
        self.api.add_uploader_gpg_keys_to_pocket(
            "name", "series", "distribution", ["key1", "key2"])
        self.assertCallEqual(
            "AddUploaderGPGKeysToPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "gpg_keys.1": "key1", "gpg_keys.2": "key2"})

    def test_remove_uploader_gpg_keys_from_pocket(self):
        """
        L{API.remove_uploader_gpg_keys_from_pocket} makes a
        C{RemoveUploaderGPGKeysFromPocket} call with the C{run_query} function.
        """
        self.api.remove_uploader_gpg_keys_from_pocket(
            "name", "series", "distribution", ["key1", "key2"])
        self.assertCallEqual(
            "RemoveUploaderGPGKeysFromPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "gpg_keys.1": "key1", "gpg_keys.2": "key2"})

    def test_add_package_filters_to_pocket(self):
        """
        L{API.add_package_filters_to_pocket} makes an
        C{AddPackageFiltersToPocket} call with the C{run_query} function.
        """
        self.api.add_package_filters_to_pocket(
            "name", "series", "distribution", ["package1", "package2"])
        self.assertCallEqual(
            "AddPackageFiltersToPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "packages.1": "package1", "packages.2": "package2"})

    def test_remove_package_filters_from_pocket(self):
        """
        L{API.remove_package_filters_from_pocket} makes a
        C{RemovePackageFiltersFromPocket} call with the C{run_query} function.
        """
        self.api.remove_package_filters_from_pocket(
            "name", "series", "distribution", ["package1", "package2"])
        self.assertCallEqual(
            "RemovePackageFiltersFromPocket",
            {"name": "name", "series": "series",
             "distribution": "distribution",
             "packages.1": "package1", "packages.2": "package2"})

    def test_get_apt_sources(self):
        """
        L{API.get_apt_sources} makes a C{GetAPTSources} call with the
        C{run_query} function.
        """
        self.api.get_apt_sources()
        self.assertCallEqual("GetAPTSources", {})

    def test_get_apt_sources_with_names(self):
        """
        L{API.get_apt_sources} can accept a list of names.
        """
        self.api.get_apt_sources(names=["source1", "source2"])
        self.assertCallEqual("GetAPTSources",
                             {"names.1": "source1", "names.2": "source2"})

    def test_create_apt_source(self):
        """
        L{API.create_apt_source} makes a C{CreateAPTSource} call with the
        C{run_query} function.
        """
        self.api.create_apt_source("name", "deb line")
        self.assertCallEqual(
            "CreateAPTSource",
            {"name": "name", "apt_line": "deb line"})

    def test_create_apt_source_with_gpg_key(self):
        """
        L{API.create_apt_sources} accepts a GPG key name.
        """
        self.api.create_apt_source("name", "deb line", gpg_key="key")
        self.assertCallEqual(
            "CreateAPTSource",
            {"name": "name", "apt_line": "deb line", "gpg_key": "key"})

    def test_remove_apt_sources(self):
        """
        L{API.remove_apt_sources} makes a C{RemoveAPTSources} call with the
        C{run_query} function.
        """
        self.api.remove_apt_sources(names=["source1", "source2"])
        self.assertCallEqual(
            "RemoveAPTSources",
            {"names.1": "source1", "names.2": "source2"})

    def test_get_repository_profiles(self):
        """
        L{API.get_repository_profiles} makes a C{GetRepositoryProfiles} call
        with the C{run_query} function.
        """
        self.api.get_repository_profiles()
        self.assertCallEqual("GetRepositoryProfiles", {})

    def test_get_repository_profiles_with_names_as_list(self):
        """
        L{API.get_repository_profiles} makes a C{GetRepositoryProfiles} call
        with the C{run_query} function.
        """
        self.api.get_repository_profiles(names=["profile1", "profile2"])
        self.assertCallEqual("GetRepositoryProfiles",
                             {"names.1": "profile1", "names.2": "profile2"})

    def test_create_repository_profile(self):
        """
        L{API.create_repository_profile} makes a C{CreateRepositoryProfile}
        call with the C{run_query} function.
        """
        self.api.create_repository_profile("name", "description")
        self.assertCallEqual(
            "CreateRepositoryProfile",
            {"name": "name", "description": "description"})

    def test_create_repository_profile_no_description(self):
        """
        L{API.create_repository_profile} makes a C{CreateRepositoryProfile}
        call with the C{run_query} function.
        """
        self.api.create_repository_profile("name")
        self.assertCallEqual(
            "CreateRepositoryProfile", {"name": "name"})

    def test_remove_repository_profiles(self):
        """
        L{API.remove_repository_profiles} makes a C{RemoveRepositoryProfiles}
        call with the C{run_query} function.
        """
        self.api.remove_repository_profiles(names=["profile-1", "profile-2"])
        self.assertCallEqual(
            "RemoveRepositoryProfiles",
            {"names.1": "profile-1", "names.2": "profile-2"})

    def test_edit_repository_profile(self):
        """
        L{API.edit_repository_profile} makes a C{EditRepositoryProfile}
        call with the C{run_query} function.
        """
        self.api.edit_repository_profile("name", "description")
        self.assertCallEqual(
            "EditRepositoryProfile",
            {"name": "name", "description": "description"})

    def test_associate_repository_profile_tags(self):
        """
        L{API.associate_repository_profile} makes an
        C{AssociateRepositoryProfile} call with the C{run_query} function.
        """
        self.api.associate_repository_profile("name", tags=["natty", "server"])
        self.assertCallEqual(
            "AssociateRepositoryProfile",
            {"name": "name", "tags.1": "natty", "tags.2": "server"})

    def test_associate_repository_profile_all_computers(self):
        """
        L{API.associate_repository_profile} makes an
        C{AssociateRepositoryProfile} call with the C{run_query} function.
        """
        self.api.associate_repository_profile("name", all_computers=True)
        self.assertCallEqual(
            "AssociateRepositoryProfile",
            {"name": "name", "all_computers": "true"})

    def test_disassociate_repository_profile_tags(self):
        """
        L{API.disassociate_repository_profile} makes a
        C{DisassociateRepositoryProfile} call with the C{run_query} function.
        """
        self.api.disassociate_repository_profile("name",
                                                 tags=["natty", "server"])
        self.assertCallEqual(
            "DisassociateRepositoryProfile",
            {"name": "name", "tags.1": "natty", "tags.2": "server"})

    def test_disassociate_repository_profile_all_computers(self):
        """
        L{API.disassociate_repository_profile} makes a
        C{DisassociateRepositoryProfile} call with the C{run_query} function.
        """
        self.api.disassociate_repository_profile("name", all_computers=True)
        self.assertCallEqual(
            "DisassociateRepositoryProfile",
            {"name": "name", "all_computers": "true"})

    def test_add_pockets_to_repository_profile(self):
        """
        L{API.add_pockets_to_repository_profile} makes a
        C{AddPocketsToRepositoryProfile} call with the C{run_query} function.
        """
        self.api.add_pockets_to_repository_profile("name",
                                                   ["pocket1", "pocket2"],
                                                   "series", "distribution")
        self.assertCallEqual(
            "AddPocketsToRepositoryProfile",
            {"name": "name", "pockets.1": "pocket1", "pockets.2": "pocket2",
             "series": "series", "distribution": "distribution"})

    def test_remove_pockets_from_repository_profile(self):
        """
        L{API.remove_pockets_from_repository_profile} makes a
        C{RemovePocketsFromRepositoryProfile} call with the C{run_query}
        function.
        """
        self.api.remove_pockets_from_repository_profile("name",
                                                        ["pocket1", "pocket2"],
                                                        "series",
                                                        "distribution")
        self.assertCallEqual(
            "RemovePocketsFromRepositoryProfile",
            {"name": "name", "pockets.1": "pocket1", "pockets.2": "pocket2",
             "series": "series", "distribution": "distribution"})

    def test_add_apt_sources_to_repository_profile(self):
        """
        L{API.add_apt_sources_to_repository_profile} makes a
        C{AddAPTSourcesToRepositoryProfile} call with the C{run_query}
        function.
        """
        self.api.add_apt_sources_to_repository_profile(
            "name", ["source1", "source2"])
        self.assertCallEqual(
            "AddAPTSourcesToRepositoryProfile",
            {"name": "name",
             "apt_sources.1": "source1", "apt_sources.2": "source2"})

    def test_remove_apt_sources_from_repository_profile(self):
        """
        L{API.remove_apt_sources_from_repository_profile} makes a
        C{RemoveAPTSourcesFromRepositoryProfile} call with the C{run_query}
        function.
        """
        self.api.remove_apt_sources_from_repository_profile(
            "name", ["source1", "source2"])
        self.assertCallEqual(
            "RemoveAPTSourcesFromRepositoryProfile",
            {"name": "name",
             "apt_sources.1": "source1", "apt_sources.2": "source2"})

    def test_get_saved_searches(self):
        """
        L{API.get_saved_searches} makes a C{GetSavedSearches} call with the
        C{run_query} function.
        """
        self.api.get_saved_searches()
        self.assertCallEqual(
            "GetSavedSearches",
            {})

    def test_get_saved_searches_with_limit(self):
        """
        L{API.get_saved_searches} makes a C{GetSavedSearches} call with the
        C{run_query} function with a C{limit} parameter.
        """
        self.api.get_saved_searches(limit=10)
        self.assertCallEqual(
            "GetSavedSearches",
            {"limit": "10"})

    def test_get_saved_searches_with_offset(self):
        """
        L{API.get_saved_searches} makes a C{GetSavedSearches} call with the
        C{run_query} function with an C{offset} parameter.
        """
        self.api.get_saved_searches(offset=10)
        self.assertCallEqual(
            "GetSavedSearches",
            {"offset": "10"})

    def test_get_saved_searches_with_limit_and_offset(self):
        """
        L{API.get_saved_searches} makes a C{GetSavedSearches} call with the
        C{run_query} function with a C{limit} and an C{offset} parameter.
        """
        self.api.get_saved_searches(limit=10, offset=10)
        self.assertCallEqual(
            "GetSavedSearches",
            {"limit": "10", "offset": "10"})

    def test_create_saved_search(self):
        """
        L{API.create_saved_search} makes a C{CreateSavedSearch} call with the
        C{run_query} function and includes the C{Name} parameter.
        """
        self.api.create_saved_search("title", "id:1", name="name")
        self.assertCallEqual(
            "CreateSavedSearch",
            {"name": "name", "title": "title", "search": "id:1"})

    def test_create_saved_search_no_name(self):
        """
        L{API.create_saved_search} makes a C{CreateSavedSearch} call with the
        C{run_query} function and does not include the optional C{Name}
        parameter.
        """
        self.api.create_saved_search("title", "id:1")
        self.assertCallEqual("CreateSavedSearch",
                             {"title": "title", "search": "id:1"})

    def test_remove_saved_search(self):
        """
        L{API.remove_saved_search} makes a C{RemoveSavedSearch} call with the
        C{run_query} function and includes the C{Name} parameter.
        """
        self.api.remove_saved_search("name")
        self.assertCallEqual("RemoveSavedSearch", {"name": "name"})

    def test_edit_saved_search(self):
        """
        L{API.edit_saved_search} makes an C{EditSavedSearch} call with the
        C{run_query} function.
        """
        self.api.edit_saved_search("name")
        self.assertCallEqual("EditSavedSearch", {"name": "name"})

    def test_edit_saved_search_with_title(self):
        """
        L{API.edit_saved_search} makes an C{EditSavedSearch} call with the
        C{run_query} function and includes the C{title} parameter.
        """
        self.api.edit_saved_search("name", title="title")
        self.assertCallEqual("EditSavedSearch",
                             {"name": "name", "title": "title"})

    def test_edit_saved_search_with_search(self):
        """
        L{API.edit_saved_search} makes an C{EditSavedSearch} call with the
        C{run_query} function and includes the C{Search} parameter.
        """
        self.api.edit_saved_search("name", search="search")
        self.assertCallEqual("EditSavedSearch",
                             {"name": "name", "search": "search"})

    def test_edit_saved_search_with_title_and_search(self):
        """
        L{API.edit_saved_search} makes an C{EditSavedSearch} call with the
        C{run_query} function and includes the C{Title} and C{Search}
        parameters.
        """
        self.api.edit_saved_search("name", title="title", search="search")
        self.assertCallEqual("EditSavedSearch",
                             {"name": "name",
                              "title": "title",
                              "search": "search"})

    def test_accept_pending_computers(self):
        """
        L{API.accept_pending_computers} makes a C{AcceptPendingComputers} call
        with the C{run_query} function.
        """
        self.api.accept_pending_computers([1, 2, 3])
        self.assertCallEqual(
            "AcceptPendingComputers",
            {"computer_ids.1": "1", "computer_ids.2": "2",
             "computer_ids.3": "3"})

    def test_reject_pending_computers(self):
        """
        L{API.reject_pending_computers} makes a C{RejectPendingComputers} call
        with the C{run_query} function.
        """
        self.api.reject_pending_computers([1, 2, 3])
        self.assertCallEqual(
            "RejectPendingComputers",
            {"computer_ids.1": "1", "computer_ids.2": "2",
             "computer_ids.3": "3"})

    def test_remove_computers(self):
        """
        L{API.remove_computers} makes a C{RemoveComputers} call with the
        C{run_query} function.
        """
        self.api.remove_computers([1, 2, 3])
        self.assertCallEqual(
            "RemoveComputers",
            {"computer_ids.1": "1", "computer_ids.2": "2",
             "computer_ids.3": "3"})

    def test_invite_administrator(self):
        """
        L{API.invite_administrator} makes a C{InviteAdministrator} call with a
        C{run_query} function.
        """
        self.api.invite_administrator("blah", "blah@example.com")
        self.assertCallEqual(
            "InviteAdministrator",
            {"name": "blah", "email": "blah@example.com"})

    def test_invite_administrator_with_optional_params(self):
        """
        L{API.invite_administrator} makes a C{InviteAdministrator} call with a
        C{run_query} function, testing with optional parameters.
        """
        self.api.invite_administrator(
            "blah", "blah@example.com", "SuperRole")
        self.assertCallEqual(
            "InviteAdministrator",
            {"name": "blah", "email": "blah@example.com", "role": "SuperRole"})

    def test_get_administrators(self):
        """
        L{API.get_administrators} makes a C{GetAdministrators} call with a
        C{run_query} function.
        """
        self.api.get_administrators()
        self.assertCallEqual("GetAdministrators", {})

    def test_disable_administrator(self):
        """
        L{API.disable_administrator} makes a C{DisableAdministrator} call with
        a C{run_query} function.
        """
        self.api.disable_administrator("blah@example.com")
        self.assertCallEqual("DisableAdministrator",
                             {"email": "blah@example.com"})

    def test_create_upgrade(self):
        """
        L{API.create_upgrade_profile} makes a C{CreateUpgradeProfile}
        call with the C{run_query} function.
        """
        self.api.create_upgrade_profile("name", every="week", at_minute=0)
        self.assertCallEqual("CreateUpgradeProfile",
                             {"name": "name", "every": "week",
                              "at_minute": "0"})

    def test_create_upgrade_with_security_upgrade(self):
        """
        L{API.create_upgrade_profile} makes a C{CreateUpgradeProfile}
        call with the C{run_query} function, with the optional
        security_upgrade parameter.
        """
        self.api.create_upgrade_profile(
            "name", every="week", at_minute=0, security_upgrade=True)
        self.assertCallEqual("CreateUpgradeProfile",
                             {"name": "name", "every": "week",
                              "at_minute": "0", "security_upgrade": "true"})

    def test_create_upgrade_with_access_group(self):
        """
        L{API.create_upgrade_profile} makes a C{CreateUpgradeProfile}
        call with the C{run_query} function, with the optional
        access_group parameter.
        """
        self.api.create_upgrade_profile(
            "name", every="week", at_minute=0, access_group="my-access-group")
        self.assertCallEqual(
            "CreateUpgradeProfile",
            {"name": "name", "every": "week", "at_minute": "0",
             "access_group": "my-access-group"})

    def test_edit_upgrade_no_optional(self):
        """
        L{API.edit_upgrade_profile} makes a C{EditUpgradeProfile}
        call with the C{run_query} function, without the optional
        security_upgrade parameter.
        """
        self.api.edit_upgrade_profile("name", every="day", at_minute=0)
        self.assertCallEqual("EditUpgradeProfile",
                             {"name": "name",
                              "at_minute": "0",
                              "every": "day"})

    def test_edit_upgrade_with_optional(self):
        """
        L{API.edit_upgrade_profile} makes a C{EditUpgradeProfile}
        call with the C{run_query} function, with the optional
        security_upgrade parameter.
        """
        self.api.edit_upgrade_profile("name", every="day",
                                           at_minute=0,
                                           security_upgrade=True)
        self.assertCallEqual("EditUpgradeProfile",
                             {"name": "name",
                              "every": "day",
                              "at_minute": "0",
                              "security_upgrade": "true"})

    def test_remove_upgrade(self):
        """
        L{API.remove_upgrade_profile} makes a C{RemoveUpgradeProfile}
        call with the C{run_query} function.
        """
        self.api.remove_upgrade_profile("name")
        self.assertCallEqual("RemoveUpgradeProfile", {"name": "name"})

    def test_get_all_upgrade(self):
        """
        L{API.get_upgrade_profiles} makes a C{GetUpgradeProfiles}
        call with the C{run_query} function, without passing any parameters.
        """
        self.api.get_upgrade_profiles()
        self.assertCallEqual("GetUpgradeProfiles", {})

    def test_get_security_upgrade(self):
        """
        L{API.get_upgrade_profiles} makes a C{GetUpgradeProfiles}
        call with the C{run_query} function, passing the optional parameter
        C{upgrade_type}.
        """
        # We can assume "all" would work, too.
        self.api.get_upgrade_profiles("security")
        self.assertCallEqual("GetUpgradeProfiles",
                             {"upgrade_type": "security"})

    def test_associate_upgrade_tags(self):
        """
        L{API.associate_upgrade_profile} makes an
        C{AssociateUpgradeProfile} call with the C{run_query} function, passing
        tags in parameter.
        """
        self.api.associate_upgrade_profile("name",
                                                tags=["natty", "server"])
        self.assertCallEqual(
            "AssociateUpgradeProfile",
            {"name": "name", "tags.1": "natty", "tags.2": "server"})

    def test_associate_upgrade_all_computers(self):
        """
        L{API.associate_upgrade_profile} makes an
        C{AssociateUpgradeProfile} call with the C{run_query} function. The
        call passes all_computers instead of tags.
        """
        self.api.associate_upgrade_profile("name", all_computers=True)
        self.assertCallEqual(
            "AssociateUpgradeProfile",
            {"name": "name", "all_computers": 'true'})

    def test_disassociate_upgrade_tags(self):
        """
        L{API.disassociate_upgrade_profile} makes a
        C{DisassociateUpgradeProfile} call with the C{run_query} function,
        passing tags in parameter.
        """
        self.api.disassociate_upgrade_profile("name",
                                                   tags=["natty", "server"])
        self.assertCallEqual(
            "DisassociateUpgradeProfile",
            {"name": "name", "tags.1": "natty", "tags.2": "server"})

    def test_disassociate_upgrade_all_computers(self):
        """
        L{API.disassociate_upgrade_profile} makes a
        C{DisassociateUpgradeProfile} call with the C{run_query} function. The
        call passes all_computers instead of tags.
        """
        self.api.disassociate_upgrade_profile("name", all_computers=True)
        self.assertCallEqual(
            "DisassociateUpgradeProfile",
            {"name": "name", "all_computers": 'true'})

    def test_create_role(self):
        """
        L{API.create_role} calls the C{CreateRole} API method.
        """
        self.api.create_role("Role")
        self.assertCallEqual("CreateRole", {"name": "Role"})

    def test_create_role_with_description(self):
        """
        L{API.create_role} optionally passes the description parameter
        to the C{CreateRole} API.
        """
        self.api.create_role("Role", description="I'm a role")
        self.assertCallEqual("CreateRole", {"name": "Role",
                                            "description": "I'm a role"})

    def test_add_permissions_to_role(self):
        """
        L{API.add_permissions_to_role} calls the C{AddPermissionsToRole} API.
        """
        self.api.add_permissions_to_role("Role", permissions=["ExecuteScript"])
        self.assertCallEqual("AddPermissionsToRole",
                             {"name": "Role",
                              "permissions.1": "ExecuteScript"})

    def test_remove_role(self):
        """
        L{API.remove_role} calls the C{RemoveRole} API method.
        """
        self.api.remove_role("Role")
        self.assertCallEqual("RemoveRole", {"name": "Role"})

    def test_add_persons_to_role(self):
        """
        L{API.add_persons_to_role} calls the C{AddPersonsToRole} API.
        """
        self.api.add_persons_to_role("Role", persons=["john@example.com"])
        self.assertCallEqual("AddPersonsToRole",
                             {"name": "Role",
                              "persons.1": "john@example.com"})

    def test_add_access_groups_to_role(self):
        """
        L{API.add_access_groups_to_role} calls the C{AddAccesssGroupsToRole}
        API.
        """
        self.api.add_access_groups_to_role("Role", access_groups=["Group"])
        self.assertCallEqual("AddAccessGroupsToRole",
                             {"name": "Role",
                              "access_groups.1": "Group"})

    def test_remove_access_groups_from_role(self):
        """
        L{API.remove_access_groups_from_role} calls the
        C{RemoveAccessGroupsFromRole} API.
        """
        self.api.remove_access_groups_from_role("Role",
                                                access_groups=["Group"])
        self.assertCallEqual("RemoveAccessGroupsFromRole",
                             {"name": "Role",
                              "access_groups.1": "Group"})
